Package: REMP
Type: Package
Title: Repetitive Element Methylation Prediction
Version: 1.37.0
Authors@R: c(person("Yinan","Zheng", role = c("aut","cre"), email = "y-zheng@northwestern.edu"),person("Lei","Liu", role = c("aut"), email = "lei.liu@northwestern.edu "), person("Wei","Zhang", role = c("aut"), email = "wei.zhang1@northwestern.edu"), person("Warren","Kibbe", role = c("aut"), email = "warren.kibbe@nih.gov"), person("Lifang","Hou", role = c("aut","cph"), email = "l-hou@northwestern.edu"))
Description: Machine learning-based tools to predict DNA methylation of locus-specific repetitive elements (RE) by learning surrounding genetic and epigenetic information. These tools provide genomewide and single-base resolution of DNA methylation prediction on RE that are difficult to measure using array-based or sequencing-based platforms, which enables epigenome-wide association study (EWAS) and differentially methylated region (DMR) analysis on RE. 
License: GPL-3
Depends: R (>= 3.6), SummarizedExperiment(>= 1.1.6), minfi (>= 1.22.0)
Imports: readr, rtracklayer, graphics, stats, utils, methods, settings,
        BiocGenerics, S4Vectors, Biostrings, GenomicRanges, IRanges,
        Seqinfo, BiocParallel, doParallel, parallel, foreach, caret,
        kernlab, ranger, BSgenome, AnnotationHub, org.Hs.eg.db, impute,
        iterators
Suggests: IlluminaHumanMethylation450kanno.ilmn12.hg19,
        IlluminaHumanMethylationEPICanno.ilm10b2.hg19,
        BSgenome.Hsapiens.UCSC.hg19, BSgenome.Hsapiens.UCSC.hg38,
        knitr, rmarkdown, minfiDataEPIC
Collate: REMParcel-class.R REMProduct-class.R generics.R
        REMParcel-methods.R REMProduct-methods.R options.R utils.R
        REMPtools.R getGM12878.R grooMethy.R initREMP.R rempTemplate.R
        remp.R remprofile.R DemoData.R onAttach.R REMP-package.R
URL: https://github.com/YinanZheng/REMP
BugReports: https://github.com/YinanZheng/REMP/issues
Encoding: UTF-8
LazyData: true
biocViews: DNAMethylation, Microarray, MethylationArray, Sequencing,
        GenomeWideAssociation, Epigenetics, Preprocessing,
        MultiChannel, TwoChannel, DifferentialMethylation,
        QualityControl, DataImport
RoxygenNote: 7.1.2
git_url: https://git.bioconductor.org/packages/REMP
git_branch: devel
git_last_commit: 3f67cb8
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-29
NeedsCompilation: no
Packaged: 2026-04-29 23:56:13 UTC; biocbuild
Author: Yinan Zheng [aut, cre],
  Lei Liu [aut],
  Wei Zhang [aut],
  Warren Kibbe [aut],
  Lifang Hou [aut, cph]
Maintainer: Yinan Zheng <y-zheng@northwestern.edu>
