# Generated by roxygen2: do not edit by hand

S3method("[",template)
export(GRannot)
export(fetchRMSK)
export(fetchRefSeqGene)
export(findRECpG)
export(getBackend)
export(getGM12878)
export(grooMethy)
export(initREMP)
export(remp)
export(rempTemplate)
export(remp_options)
export(remp_reset)
export(remprofile)
exportClasses(REMParcel)
exportClasses(REMProduct)
exportMethods(decodeAnnot)
exportMethods(details)
exportMethods(getILMN)
exportMethods(getParcelInfo)
exportMethods(getRE)
exportMethods(getRECpG)
exportMethods(getRefGene)
exportMethods(rempAggregate)
exportMethods(rempAnnot)
exportMethods(rempB)
exportMethods(rempCombine)
exportMethods(rempImp)
exportMethods(rempM)
exportMethods(rempQC)
exportMethods(rempStats)
exportMethods(rempTrim)
exportMethods(remplot)
exportMethods(saveParcel)
import(BiocParallel)
import(S4Vectors)
import(caret)
import(methods)
importFrom(AnnotationHub,AnnotationHub)
importFrom(AnnotationHub,getAnnotationHubOption)
importFrom(AnnotationHub,setAnnotationHubOption)
importFrom(BSgenome,getSeq)
importFrom(BiocGenerics,mget)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,vmatchPattern)
importFrom(GenomicRanges,"end<-")
importFrom(GenomicRanges,"start<-")
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,findOverlaps)
importFrom(GenomicRanges,granges)
importFrom(GenomicRanges,makeGRangesListFromFeatureFragments)
importFrom(GenomicRanges,promoters)
importFrom(GenomicRanges,ranges)
importFrom(GenomicRanges,seqnames)
importFrom(GenomicRanges,shift)
importFrom(GenomicRanges,start)
importFrom(GenomicRanges,strand)
importFrom(IRanges,CharacterList)
importFrom(IRanges,IRanges)
importFrom(IRanges,IRangesList)
importFrom(IRanges,NumericList)
importFrom(IRanges,subsetByOverlaps)
importFrom(Seqinfo,"seqlevels<-")
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(SummarizedExperiment,rowRanges)
importFrom(doParallel,registerDoParallel)
importFrom(foreach,registerDoSEQ)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(impute,impute.knn)
importFrom(iterators,idiv)
importFrom(iterators,nextElem)
importFrom(kernlab,predict)
importFrom(minfi,GenomicRatioSet)
importFrom(minfi,IlluminaMethylationAnnotation)
importFrom(minfi,MethylSet)
importFrom(minfi,RatioSet)
importFrom(minfi,getAnnotation)
importFrom(minfi,getBeta)
importFrom(minfi,getM)
importFrom(org.Hs.eg.db,org.Hs.egREFSEQ2EG)
importFrom(org.Hs.eg.db,org.Hs.egSYMBOL)
importFrom(parallel,detectCores)
importFrom(ranger,importance)
importFrom(ranger,ranger)
importFrom(readr,col_character)
importFrom(readr,col_integer)
importFrom(readr,cols_only)
importFrom(readr,read_delim)
importFrom(rtracklayer,liftOver)
importFrom(settings,options_manager)
importFrom(settings,reset)
importFrom(settings,stop_if_reserved)
importFrom(stats,density)
importFrom(stats,na.pass)
importFrom(stats,setNames)
importFrom(utils,data)
importFrom(utils,download.file)
importFrom(utils,packageVersion)
importFrom(utils,read.table)
